<?php
declare(strict_types=1);

namespace FiloBlu\Refilo\Remote;

use Exception;
use Magento\Framework\Exception\NoSuchEntityException;

/**
 * Class AbstractStoreBasedIndexer
 * @package FiloBlu\Refilo\Remote
 */
abstract class AbstractStoreBasedIndexer extends AbstractIndexer
{

    /**
     * @param array $ids
     * @return mixed|void
     * @throws NoSuchEntityException
     */
    public function executeAction(array $ids)
    {
        // Profiler::start( __METHOD__);

        try {
            $indexTemplate = $this->getGetRemoteIndexTemplate();
        } catch (Exception $exception) {
            $this->logger->error($exception->getMessage(), ['exception' => $exception]);
            return;
        }

        $previousStore = $this->getStoreManager()->getStore();

        foreach ($this->getStoreManager()->getStores() as $store) {


            if (!$store->getIsActive()) {
                continue;
            }

            // Profiler::start( 'process ' . $store->getCode());

            $collection = $indexTemplate->withValue('%s', $store->getCode())->render();

            $this->getStoreManager()->setCurrentStore($store);

            if ($this->getIndexerHelper()->indexerCanDelete()) {
                $this->getConnector()->delete($this->getEntitiesToRemove($ids), $collection);
            }

            $arguments = $this->dataObjectFactory->create([
                'data' => [
                    self::ARGUMENT_COLLECTION => $collection
                ]
            ]);

            iterator_to_array(
                $this->entityProviderFactory->create()
                    ->forStore($store)
                    ->withIds($ids)
                    ->attachReaderHandler($this, $arguments)
            );

            // Profiler::stop( 'process ' . $store->getCode());

        }

        $this->getStoreManager()->setCurrentStore($previousStore);

        // Profiler::stop( __METHOD__);
    }
}
